"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var crypto = require("crypto");
var request = require("request");
var channel_1 = require("./channel");
var logger_1 = require("./logger");
var WeChat = /** @class */ (function (_super) {
    __extends(WeChat, _super);
    function WeChat(params) {
        return _super.call(this, {
            homedir: params.homedir,
            uid: params.uid,
            platform: 'wechat'
        }) || this;
    }
    WeChat.prototype.execute = function (body, parameters, handler) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            logger_1.Logger.init(_this.params).then(function (logger) {
                logger.incoming(body);
                handler(_this.params.uid, body, _this.params.platform, function (message) {
                    var messages = [];
                    console.log(message);
                    if (!Array.isArray(message)) {
                        if (message) {
                            messages.push(message);
                        }
                        messages.push({
                            type: "image",
                            picUrl: "https://assets.recime.io/4a62ca429c2bfb0040ab1a018e92aebb/giphy.gif"
                        });
                    }
                    ;
                    _this.sendMessage(0, body.FromUserName, messages, logger).then(function () {
                        resolve();
                    });
                }, function (err) {
                    resolve(err.message);
                });
            });
        });
    };
    WeChat.prototype.sendMessage = function (index, toUser, messages, logger) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var message = messages[index];
            var reqData = {
                touser: toUser,
                msgtype: message.type,
            };
            switch (message.type) {
                case "text": {
                    reqData.text = {
                        content: message.content
                    };
                    break;
                }
                // case "image": {
                //     return this.uploadImage(message.picUrl).then(()=>{
                //         if (index + 1 < messages.length) {
                //             this.sendMessage(++index, toUser, messages, logger).then(resolve);
                //         }
                //         this.postMessage(reqData).then(resolve);
                //     });
                //     // reqData.image = {
                //     //     media_id: message.content
                //     // };
                // }
                default: {
                    reqData.msgtype = "text";
                    reqData.text = {
                        content: message.text || "Message type not supported."
                    };
                }
            }
            ;
            if (index + 1 < messages.length) {
                _this.sendMessage(++index, toUser, messages, logger).then(resolve);
            }
            _this.postMessage(reqData).then(resolve);
        });
    };
    WeChat.prototype.postMessage = function (data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.accessToken().then(function (accessToken) {
                request({
                    url: "https://api.wechat.com/cgi-bin/message/custom/send?access_token=" + accessToken,
                    json: true,
                    method: "POST",
                    headers: {
                        "content-type": "application/x-www-form-urlencoded",
                    },
                    body: JSON.stringify(data)
                }, function (err, response, body) {
                    console.log(body);
                    if (err) {
                        return resolve(err.message);
                    }
                    if (response.statusCode < 200 && response.statusCode > 299) {
                        return resolve();
                    }
                    resolve();
                });
            })
                .catch(reject);
        });
    };
    WeChat.prototype.accessToken = function () {
        return new Promise(function (resolve, reject) {
            request.get("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + (process.env.WECHAT_APPID || process.env.RECIME_WECHAT_APPID) + "&secret=" + (process.env.WECHAT_APPSECRET || process.env.RECIME_WECHAT_APPSECRET), {
                "Accept": "application/json"
            }, function (err, response, body) {
                if (response.statusCode < 200 && response.statusCode > 299) {
                    return reject();
                }
                var accessToken = JSON.parse(response.body).access_token;
                resolve(accessToken);
            });
        });
    };
    WeChat.prototype.uploadImage = function (picUrl) {
        var _this = this;
        var parsed = require("url").parse(picUrl).path;
        return new Promise(function (resolve, reject) {
            _this.accessToken().then(function (accessToken) {
                var r = request({
                    url: "https://api.weixin.qq.com/cgi-bin/media/upload?access_token=" + accessToken + "&type=image",
                    method: "POST"
                }, function (err, response, body) {
                    resolve();
                });
            });
        });
        // });
    };
    WeChat.prototype.verify = function (query) {
        var accessToken = process.env.WECHAT_ACCESS_TOKEN || process.env.RECIME_WECHAT_ACCESS_TOKEN;
        var token = [query['timestamp'], query['nonce'], accessToken].sort().join("");
        var hash = crypto.createHash('sha1').update(token).digest('hex');
        return hash === query['signature'];
    };
    return WeChat;
}(channel_1.Channel));
exports.WeChat = WeChat;
