import fs = require('fs');
import path = require("path");
import util = require('util');
import crypto = require('crypto');
import FlakeIdGen = require('flake-idgen')
import intformat = require('biguint-format')
import request = require('request');
import qs = require("querystring");


import { Channel } from './channel';
import { Logger } from "./logger";

export class WeChat extends Channel {
    private logger: any;

    constructor(params: any) {
        super({
            homedir: params.homedir,
            uid: params.uid,
            platform: 'wechat'
        });
    }

    execute(body, parameters, handler){
        return new Promise((resolve, reject)=>{
            Logger.init(this.params).then((logger) => {
                logger.incoming(body);
 

                handler(this.params.uid, body, this.params.platform, (message)=>{
                    const messages = [];
                    console.log(message);
                    if (!Array.isArray(message)) {
                        if (message){
                            messages.push(message);
                        }
                        
                        messages.push({
                            type: "image",
                            picUrl: "https://assets.recime.io/4a62ca429c2bfb0040ab1a018e92aebb/giphy.gif"
                        });
                    };
                    this.sendMessage(0, body.FromUserName,  messages, logger).then (()=>{
                        resolve();
                    });
                }, (err)=>{
                    resolve(err.message);
                });
            });
        });
    }
    
    sendMessage(index, toUser, messages, logger){
        return new Promise((resolve, reject) => { 
            const message = messages[index];

            const reqData: any = {
                touser: toUser,
                msgtype: message.type,
            };

            switch (message.type) {
                case "text": {
                    reqData.text = {
                        content: message.content
                    };
                    break;
                }
                // case "image": {
                //     return this.uploadImage(message.picUrl).then(()=>{
                //         if (index + 1 < messages.length) {
                //             this.sendMessage(++index, toUser, messages, logger).then(resolve);
                //         }
                //         this.postMessage(reqData).then(resolve);
                //     });

                //     // reqData.image = {
                //     //     media_id: message.content
                //     // };
                // }
                default: {
                    reqData.msgtype = "text";
                    reqData.text = {
                        content: message.text || "Message type not supported."
                    };
                }
            };
            if (index + 1 < messages.length) {
                this.sendMessage(++index, toUser, messages, logger).then(resolve);
            } 
            this.postMessage(reqData).then(resolve);
        });
    }

    postMessage(data){
        return new Promise((resolve, reject) => {
            this.accessToken().then((accessToken)=> {
                request({
                    url: `https://api.wechat.com/cgi-bin/message/custom/send?access_token=${accessToken}`,
                    json: true,
                    method: "POST",
                    headers: {
                        "content-type": "application/x-www-form-urlencoded",
                    },
                    body: JSON.stringify(data)
                }, (err, response, body) => {
                    console.log(body);
                    if (err) {
                        return resolve(err.message);
                    }
                    if (response.statusCode < 200 && response.statusCode > 299) {
                        return resolve();
                    }
                    resolve();
                });
            })
            .catch(reject);
        });
    }

    accessToken(){
        return new Promise((resolve, reject)=>{
            request.get(`https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=${process.env.WECHAT_APPID || process.env.RECIME_WECHAT_APPID}&secret=${process.env.WECHAT_APPSECRET || process.env.RECIME_WECHAT_APPSECRET}`, {
                "Accept": "application/json"
            }, (err, response, body) => {
                if (response.statusCode < 200 && response.statusCode > 299) {
                    return reject();
                }
                const accessToken = JSON.parse(response.body).access_token;
                resolve(accessToken);
            });
        });
    }

    uploadImage(picUrl){
        var parsed = require("url").parse(picUrl).path;

        return new Promise((resolve, reject) => {
            this.accessToken().then((accessToken)=>{
                    const r  = request({
                        url: `https://api.weixin.qq.com/cgi-bin/media/upload?access_token=${accessToken}&type=image`,
                        method : "POST"
                    }, (err, response, body)=>{
                        resolve();
                    }); 
                });
            });
        // });
    }

    verify(query){
        const accessToken = process.env.WECHAT_ACCESS_TOKEN || process.env.RECIME_WECHAT_ACCESS_TOKEN;
        const token = [query['timestamp'], query['nonce'], accessToken].sort().join("");
        const hash = crypto.createHash('sha1').update(token).digest('hex');
        return hash === query['signature'];
    }
}